@@ -21,7 +21,7 @@ from .utils import (
     seconds_until,
     convert_true_false,
     convert_to_seconds,
-    set_sigkill,
+    set_sigterm,
     get_speaker,
 )
 
@@ -49,9 +49,9 @@ def get_playlist(speaker, name):
     return None
 
 
-def print_playlist_header(playlist_name):
+def print_list_header(prefix, name):
     spacer = "  "
-    title = "Sonos Playlist: {}".format(playlist_name)
+    title = "{} {}".format(prefix, name)
     underline = "=" * (len(title))
     print(spacer + title)
     print(spacer + underline)
@@ -81,6 +81,34 @@ def print_tracks(tracks):
     return True
 
 
+def print_albums(albums, omit_first=False):
+    item_number = 1
+    for album in albums:
+        try:
+            artist = album.creator
+        except:
+            artist = ""
+        try:
+            title = album.title
+        except:
+            title = ""
+        if item_number == 1 and omit_first:
+            omit_first = False
+        else:
+            print("{:7d}: Album: {} | Artist: {}".format(item_number, title, artist))
+            item_number += 1
+    return True
+
+
+def print_artists(artists):
+    item_number = 1
+    for artist in artists:
+        artist_name = artist.title
+        print("{:7d}: {}".format(item_number, artist_name))
+        item_number += 1
+    return True
+
+
 # Action processing functions
 @zero_or_one_parameter
 def on_off_action(speaker, action, args, soco_function, use_local_speaker_list):
@@ -136,9 +164,7 @@ def list_numbered_things(speaker, action, args, soco_function, use_local_speaker
     if soco_function in [
         "get_sonos_favorites",
         "get_favorite_radio_stations",
-        "get_albums",
-        "get_artists",
-        "get_tracks",
+        # "get_tracks",
     ]:
         things = getattr(speaker.music_library, soco_function)(complete_result=True)
     else:
@@ -212,7 +238,7 @@ def print_info(speaker, action, args, soco_function, use_local_speaker_list):
     output = getattr(speaker, soco_function)()
     for item in sorted(output):
         if item not in ["metadata", "uri", "album_art"]:
-            print("  {}: {}".format(item, output[item]))
+            print("  {}: {}".format(item.capitalize(), output[item]))
     return True
 
 
@@ -535,7 +561,7 @@ def list_playlist_tracks(speaker, action, args, soco_function, use_local_speaker
     playlist = get_playlist(speaker, args[0])
     if playlist:
         print()
-        print_playlist_header(playlist.title)
+        print_list_header("Sonos Playlist:", playlist.title)
         tracks = speaker.music_library.browse_by_idstring(
             "sonos_playlists", playlist.item_id, max_items=sonos_max_items
         )
@@ -753,7 +779,7 @@ def list_all_playlist_tracks(
     playlists = speaker.get_sonos_playlists(complete_result=True)
     print()
     for playlist in playlists:
-        print_playlist_header(playlist.title)
+        print_list_header("Sonos Playlist:", playlist.title)
         tracks = speaker.music_library.browse_by_idstring(
             "sonos_playlists", playlist.item_id
         )
@@ -797,7 +823,7 @@ def wait_stopped_for(speaker, action, args, soco_function, use_local_speaker_lis
     while True:
         try:
             # ToDo: Remove temporary fix for CTRL-C not exiting
-            set_sigkill(True)
+            set_sigterm(True)
             event = sub.events.get(timeout=1.0)
             if event.variables["transport_state"] != "PLAYING":
                 sub.unsubscribe()
@@ -829,10 +855,10 @@ def wait_stopped_for(speaker, action, args, soco_function, use_local_speaker_lis
                         )
                     )
                     time.sleep(poll_interval)
-                set_sigkill(False)
+                set_sigterm(False)
                 return True
         except:
-            set_sigkill(False)
+            set_sigterm(False)
             pass
 
 
@@ -857,6 +883,125 @@ def wait_start(speaker, action, args, soco_function, use_local_speaker_list):
             pass
 
 
+@one_parameter
+def search_artists(speaker, action, args, soco_function, use_local_speaker_list):
+    ml = speaker.music_library
+    name = args[0]
+    artists = ml.get_music_library_information(
+        "artists", search_term=name, complete_result=True
+    )
+    for artist in artists:
+        print()
+        print_list_header("Sonos Music Library Albums including Artist:", artist.title)
+        albums = ml.get_music_library_information(
+            "artists", subcategories=[artist.title], max_items=sonos_max_items
+        )
+        print_albums(albums, omit_first=True)  # Omit the first (empty) entry
+        print()
+        # ToDo: Debating whether to include lists of all the tracks that feature the artist...
+        # print_list_header("Sonos Music Library Tracks with Artist:", artist.title)
+        # tracks = ml.search_track(artist.title)
+        # # tracks = ml.get_music_library_information("artists", subcategories=[name, ""], complete_result=True)
+        # print_tracks(tracks)
+        # print()
+    return True
+
+
+@zero_parameters
+def list_artists(speaker, action, args, soco_function, use_local_speaker_list):
+    ml = speaker.music_library
+    artists = ml.get_artists(complete_result=True)
+    print()
+    print_list_header("Sonos Music Library Artists", "")
+    print_artists(artists)
+    print()
+    return True
+
+
+@zero_parameters
+def list_albums(speaker, action, args, soco_function, use_local_speaker_list):
+    ml = speaker.music_library
+    artists = ml.get_albums(complete_result=True)
+    print()
+    print_list_header("Sonos Music Library Albums", "")
+    print_albums(artists)
+    print()
+    return True
+
+
+@one_parameter
+def search_albums(speaker, action, args, soco_function, use_local_speaker_list):
+    ml = speaker.music_library
+    name = args[0]
+    albums = ml.get_music_library_information(
+        "albums", search_term=name, complete_result=True
+    )
+    if len(albums):
+        print()
+        print_list_header("Sonos Music Library Album Search:", name)
+        print_albums(albums)
+        print()
+    return True
+
+
+@one_parameter
+def search_tracks(speaker, action, args, soco_function, use_local_speaker_list):
+    ml = speaker.music_library
+    name = args[0]
+    tracks = ml.get_music_library_information(
+        "tracks", search_term=name, complete_result=True
+    )
+    if len(tracks):
+        print()
+        print_list_header("Sonos Music Library Track Search:", name)
+        print_tracks(tracks)
+        print()
+    return True
+
+
+@one_parameter
+def search_library(speaker, action, args, soco_function, use_local_speaker_list):
+    search_artists(speaker, action, args, soco_function, use_local_speaker_list)
+    search_albums(speaker, action, args, soco_function, use_local_speaker_list)
+    search_tracks(speaker, action, args, soco_function, use_local_speaker_list)
+    return True
+
+
+@one_parameter
+def tracks_in_album(speaker, action, args, soco_function, use_local_speaker_list):
+    ml = speaker.music_library
+    name = args[0]
+    # tracks = ml.get_music_library_information("tracks", subcategories=[name], max_items=sonos_max_items)
+    albums = ml.get_music_library_information(
+        "albums", search_term=name, complete_result=True
+    )
+    logging.info("Found {} album(s) matching '{}'".format(len(albums), name))
+    print(albums)
+    for album in albums:
+        tracks = ml.get_music_library_information(
+            "artists", subcategories=["", album.title], complete_result=True
+        )
+        print()
+        print_list_header("Sonos Music Library Tracks in Album:", album.title)
+        print_tracks(tracks)
+        print()
+    return True
+
+
+@one_parameter
+def queue_album(speaker, action, args, soco_function, use_local_speaker_list):
+    """Add an album to the queue. If there are multiple matches, only the first
+    will be added.
+    :returns The position in the queue of the first track in the album
+    """
+    albums = speaker.music_library.get_music_library_information(
+        "albums", search_term=args[0], complete_result=True
+    )
+    for album in albums:
+        print(speaker.add_to_queue(album))
+        return True
+
+
 @one_or_more_parameters
 def if_stopped_or_playing(speaker, action, args, soco_function, use_local_speaker_list):
     """Perform the action only if the speaker is currently in the desired playback state
@@ -992,6 +1137,7 @@ actions = {
     "ql": SonosFunction(no_args_one_output, "queue_size"),
     "add_playlist_to_queue": SonosFunction(playlist_operations, "add_to_queue"),
     "add_pl_to_queue": SonosFunction(playlist_operations, "add_to_queue"),
+    "queue_playlist": SonosFunction(playlist_operations, "add_to_queue"),
     "apq": SonosFunction(playlist_operations, "add_to_queue"),
     "pause_all": SonosFunction(operate_on_all, "pause"),
     "seek": SonosFunction(seek, "seek"),
@@ -1023,9 +1169,7 @@ actions = {
     "play_favourite_radio_station": SonosFunction(play_favourite_radio, "play_uri"),
     "play_favorite_radio_station": SonosFunction(play_favourite_radio, "play_uri"),
     "pfrs": SonosFunction(play_favourite_radio, "play_uri"),
-    "albums": SonosFunction(list_numbered_things, "get_albums"),
-    "artists": SonosFunction(list_numbered_things, "get_artists"),
-    "tracks": SonosFunction(list_numbered_things, "get_tracks"),
+    # "tracks": SonosFunction(list_numbered_things, "get_tracks"),
     "alarms": SonosFunction(list_alarms, "get_alarms"),
     "libraries": SonosFunction(list_libraries, "list_library_shares"),
     "shares": SonosFunction(list_libraries, "list_library_shares"),
@@ -1045,4 +1189,20 @@ actions = {
     "wsf": SonosFunction(wait_stopped_for, ""),
     "if_stopped": SonosFunction(if_stopped_or_playing, ""),
     "if_playing": SonosFunction(if_stopped_or_playing, ""),
+    "search_library": SonosFunction(search_library, ""),
+    "sl": SonosFunction(search_library, ""),
+    "search_artists": SonosFunction(search_artists, ""),
+    "sart": SonosFunction(search_artists, ""),
+    "search_albums": SonosFunction(search_albums, ""),
+    "salb": SonosFunction(search_albums, ""),
+    "search_tracks": SonosFunction(search_tracks, ""),
+    "st": SonosFunction(search_tracks, ""),
+    "tracks_in_album": SonosFunction(tracks_in_album, ""),
+    "tia": SonosFunction(tracks_in_album, ""),
+    "list_albums": SonosFunction(list_albums, ""),
+    "albums": SonosFunction(list_albums, ""),
+    "list_artists": SonosFunction(list_artists, ""),
+    "artists": SonosFunction(list_artists, ""),
+    "queue_album": SonosFunction(queue_album, ""),
+    "qa": SonosFunction(queue_album, ""),
 }
